/**
 * @author                    Marlon Saldanha, Pablo Alles
 * @param onChangeText        callback chamado quando o conteúdo do input é alterado (se não for tagType)
 * @param onAddTag            callback chamado quando uma nova tag é adicionada (opcional)
 * @param placeholder         texto que aparecerá dentro do input como placeholder (opcional)
 * @param passwordType        informa se o input é do tipo password (opcional)
 * @param tags                array de tags adicionadas no input, serão exibidas em baixo dele (opcional)
 * @param tagType             informa se o input é do tipo tag (opcional)
 * @param errorMessage        mensagem de erro a ser passada (opcional)
 * @param value               valor inicial do inputText (opcional)
 *
 * @example                   <InputTextComponent placeholder='Nome *' tagType={true} errorMessage='Campo Obrigatório'/>
 */

import React, { useCallback, useEffect, useState } from 'react';
import {
  TextInput,
  StyleSheet,
  Text,
  View,
  TouchableOpacity,
} from 'react-native';
import FeatherIcon from 'react-native-vector-icons/Feather';
import colors from '../constants/colors';

export default function InputTextComponent({
  onChangeText,
  placeholder,
  passwordType = false,
  emailType = false,
  onAddTag,
  tags,
  tagType = false,
  errorMessage,
  value = '',
  style,
  ...props
}) {
  const [text, setText] = useState(value);
  const [inputText, setInputText] = useState('Mostrar');
  const [hidePassword, setHidePassword] = useState(false);
  const [error, setError] = useState(errorMessage);

  function handleAddTag() {
    if (text) {
      setText('');
      onAddTag(text);
    }
  }

  function toggleText() {
    if (inputText === 'Mostrar') {
      setInputText('Esconder');
      setHidePassword(true);
    } else {
      setInputText('Mostrar');
      setHidePassword(false);
    }
  }

  function handleChangeText(inputValue) {
    if (inputValue) setError(false);
    else if (errorMessage) setError(errorMessage);
    if (onChangeText) onChangeText(inputValue);
    setText(inputValue);
  }

  return (
    <View style={[styles.container, style]}>
      <TextInput
        {...props}
        value={text || value}
        placeholder={placeholder}
        style={styles.input}
        placeholderTextColor="#989898"
        secureTextEntry={passwordType && !hidePassword}
        autoCapitalize={passwordType || emailType ? 'none' : 'words'}
        keyboardType={emailType ? 'email-address' : undefined}
        onChangeText={inputValue => handleChangeText(inputValue)}
      />
      {passwordType && (
        <TouchableOpacity onPress={toggleText} style={styles.buttonContainer}>
          <Text style={styles.showText}>{inputText}</Text>
        </TouchableOpacity>
      )}
      {tagType && (
        <>
          <TouchableOpacity
            onPress={handleAddTag}
            style={styles.buttonContainer}
          >
            <FeatherIcon name="plus" size={25} color="#DF4F83"></FeatherIcon>
          </TouchableOpacity>
          <Text style={styles.tagText}>{tags.map(tag => `#${tag} `)}</Text>
        </>
      )}
      <View style={styles.errorContainer}>
        <FeatherIcon
          name="alert-circle"
          size={24}
          style={[styles.errorIcon, !error && styles.hide]}
        />
        <Text style={[styles.errorMsg, !error && styles.hide]}>
          {errorMessage}
        </Text>
      </View>
    </View>
  );
}

const styles = StyleSheet.create({
  container: {
    marginBottom: 10,
  },
  input: {
    height: 30,
    padding: 0,
    borderBottomColor: colors.primary,
    borderBottomWidth: 1,
    fontSize: 18,
    minWidth: '100%',
  },
  buttonContainer: {
    position: 'absolute',
    right: 0,
    marginRight: 8,
  },
  showText: {
    color: colors.primary,
    fontSize: 18,
    textDecorationLine: 'underline',
    fontSize: 18,
  },
  errorContainer: {
    marginTop: '1%',
    display: 'flex',
    flexDirection: 'row',
    alignItems: 'center',
  },
  hide: {
    color: colors.transparent,
  },
  errorMsg: {
    paddingLeft: '2%',
    fontSize: 16,
    fontWeight: 'bold',
    alignSelf: 'flex-start',
    color: colors.error,
  },
  errorIcon: {
    color: colors.error,
  },
  tagText: {
    marginTop: '1%',
    color: colors.textDisabled,
    fontSize: 20,
  },
});
