/**
 * Rotas para usar durante a aplicação.
 *
 * @author           João Leão, Marlon Saldanha
 * @example          <Routes/>
 * @example          <NavBar/>
 */

import React, { useEffect, useState } from 'react';
import { StyleSheet, AsyncStorage } from 'react-native';
import { NavigationContainer } from '@react-navigation/native';
import { createStackNavigator } from '@react-navigation/stack';
import { createBottomTabNavigator } from '@react-navigation/bottom-tabs';
import FontAwesomeIcon from 'react-native-vector-icons/FontAwesome';
import colors from './constants/colors';

import HomeScreen from './screens/HomeScreen';
import TutorialScreen from './screens/TutorialScreen';
import AppoioBasicScreen from './screens/AppoioBasicScreen';
import AppoioStepScreen from './screens/AppoioStepScreen';
import AppoioConfirmationScreen from './screens/AppoioConfirmationScreen';
import AppSelectionScreen from './screens/AppSelectionScreen';
import CreateAccountScreen from './screens/CreateAccountScreen';
import LoginScreen from './screens/LoginScreen';
import TutorialListScreen from './screens/TutorialListScreen';
import CuradoriaScreen from './screens/CuradoriaScreen';
import SignupAppSelectionScreen from './screens/SignupAppSelectionScreen';
import ExportUserDataScreen from './screens/ExportUserDataScreen';
import ModelScreen from './screens/ModelScreen';
import ApprovedTutorialListScreen from './screens/ApprovedTutorialListScreen';
import EditTutorialInfoScreen from './screens/EditTutorialInfoScreen';
import EditTutorialStepsScreen from './screens/EditTutorialStepsScreen';

const Stack = createStackNavigator();
const Tab = createBottomTabNavigator();

export function Appoio() {
  const [id, setId] = useState(-1);
  const [isAdmin, setIsAdmin] = useState(null);

  useEffect(() => {
    (async () => {
      let userId = -1;
      let isAdmin = null;
      try {
        userId = (await AsyncStorage.getItem('userId')) || -1;
        isAdmin = (await AsyncStorage.getItem('isAdmin')) || null;
      } catch (error) {
        console.log(error.message);
      }
      setId(parseInt(userId));
      setIsAdmin(isAdmin === 'true');
    })();
  }, []);

  return (
    <NavigationContainer>
      <Stack.Navigator screenOptions={{ headerShown: false }}>
        {id === -1 && (
          <Stack.Screen name="Authenticate" component={Authenticate} />
        )}
        {id !== -1 && isAdmin && (
          <Stack.Screen name="AdminRoot" component={AdminRoot} />
        )}
        {id !== -1 && !isAdmin && <Stack.Screen name="Root" component={Root} />}
      </Stack.Navigator>
    </NavigationContainer>
  );
}

function Authenticate() {
  return (
    <Stack.Navigator mode="modal" screenOptions={{ headerShown: false }}>
      <Stack.Screen name="LoginScreen" component={LoginScreen} />
      <Stack.Screen
        name="CreateAccountScreen"
        component={CreateAccountScreen}
      />
      <Stack.Screen
        name="SignupAppSelectionScreen"
        component={SignupAppSelectionScreen}
      />
      <Stack.Screen name="Root" component={Root} />
      <Stack.Screen name="AdminRoot" component={AdminRoot} />
    </Stack.Navigator>
  );
}

function Root() {
  return (
    <Tab.Navigator
      tabBarOptions={{
        activeTintColor: colors.primary,
        inactiveTintColor: colors.textDisabled,
        keyboardHidesTabBar: true,
        labelStyle:
          Platform.OS === 'android'
            ? androidStyles.labelStyle
            : iOSStyles.labelStyle,
        style:
          Platform.OS === 'android'
            ? androidStyles.container
            : iOSStyles.container,
      }}
    >
      <Tab.Screen
        name="Root"
        options={{
          title: 'Início',
          tabBarIcon: e => Icon({ focused: e.focused, name: 'home' }),
        }}
        component={Inicio}
      />

      <Tab.Screen
        name="SendAppoio"
        options={{
          title: 'Enviar Appoio',
          tabBarIcon: e => Icon({ focused: e.focused, name: 'plus' }),
        }}
        component={EnviarAppoio}
      />
    </Tab.Navigator>
  );
}

function Inicio() {
  return (
    <Stack.Navigator screenOptions={{ headerShown: false }}>
      <Stack.Screen name="HomeScreen" component={HomeScreen} />
      <Stack.Screen name="TutorialListScreen" component={TutorialListScreen} />
      <Stack.Screen name="Tutorial" component={TutorialScreen} />
      <Stack.Screen name="AppSelectionScreen" component={AppSelectionScreen} />
    </Stack.Navigator>
  );
}
function EnviarAppoio() {
  return (
    <Stack.Navigator screenOptions={{ headerShown: false }}>
      <Stack.Screen name="AppoioBasicScreen" component={AppoioBasicScreen} />
      <Stack.Screen name="AppoioStepScreen" component={AppoioStepScreen} />
      <Stack.Screen
        name="AppoioConfirmationScreen"
        component={AppoioConfirmationScreen}
      />
      <Stack.Screen name="Auth" component={Authenticate} />
    </Stack.Navigator>
  );
}

function AdminRoot() {
  return (
    <Tab.Navigator
      tabBarOptions={{
        activeTintColor: colors.primary,
        inactiveTintColor: colors.textDisabled,
        keyboardHidesTabBar: true,
        labelStyle:
          Platform.OS === 'android'
            ? androidStyles.labelStyle
            : iOSStyles.labelStyle,
        style:
          Platform.OS === 'android'
            ? androidStyles.container
            : iOSStyles.container,
      }}
    >
      <Tab.Screen
        name="ApprovedScreen"
        options={{
          title: 'Aprovados',
          tabBarIcon: e => Icon({ focused: e.focused, name: 'check' }),
        }}
        component={Aprovados}
      />
      <Tab.Screen
        name="Curadoria"
        options={{
          title: 'Curadoria',
          tabBarIcon: e => Icon({ focused: e.focused, name: 'filter' }),
        }}
        component={Curadoria}
      />
      <Tab.Screen
        name="Export"
        options={{
          title: 'Dados',
          tabBarIcon: e => Icon({ focused: e.focused, name: 'bar-chart' }),
        }}
        component={ExportUserDataScreen}
      />
      <Tab.Screen
        name="SendAppoioAdm"
        options={{
          title: 'Enviar Appoio',
          tabBarIcon: e => Icon({ focused: e.focused, name: 'plus' }),
        }}
        component={EnviarAppoio}
      />
    </Tab.Navigator>
  );
}

function Curadoria() {
  return (
    <Stack.Navigator screenOptions={{ headerShown: false }}>
      <Stack.Screen name="CuradoriaScreen" component={CuradoriaScreen} />
      <Stack.Screen name="TutorialScreen" component={TutorialScreen} />
    </Stack.Navigator>
  );
}
function Aprovados() {
  return (
    <Stack.Navigator screenOptions={{ headerShown: false }}>
      <Stack.Screen
        name="ApprovedTutorialListScreen"
        component={ApprovedTutorialListScreen}
      />

      <Stack.Screen
        name="EditTutorialInfoScreen"
        component={EditTutorialInfoScreen}
      />
      <Stack.Screen
        name="EditTutorialStepsScreen"
        component={EditTutorialStepsScreen}
      />
      <Stack.Screen name="TutorialScreen" component={TutorialScreen} />
    </Stack.Navigator>
  );
}

const androidStyles = StyleSheet.create({
  container: {
    height: 70,
    borderTopWidth: 1,
    borderTopColor: colors.primary,
    paddingTop: 7,
  },
  labelStyle: {
    fontSize: 15,
    marginBottom: 11,
  },
});

const iOSStyles = StyleSheet.create({
  container: {
    borderTopWidth: 1,
    borderTopColor: '#D51A5E',
    paddingTop: 5,
  },
  labelStyle: {
    fontSize: 14,
  },
});

const Icon = ({ focused, name }) => {
  return (
    <FontAwesomeIcon
      name={name}
      size={28}
      color={focused ? colors.primary : colors.textDisabled}
    />
  );
};
