/**
 * Tela de modelo com um componente de modelo
 *
 * @author            Felipe Hamester
 * @example           <ModelScreen/>
 */

import React, { useEffect, useState } from 'react';
import { StyleSheet, View, Image, Text, Alert } from 'react-native';
import colors from '../constants/colors';

import HeaderComponent from '../components/HeaderComponent';
import BodyComponent from '../components/BodyComponent';
import TutorialCard from '../components/TutorialCard';
import API from '../api';
import TabSelectionComponent from '../components/TabSelectionComponent';
import SearchBarComponent from '../components/SearchBarComponent';

export default function ApprovedTutorialListScreen({ navigation }) {
  const [selectedTab, setSelectedTab] = useState(0);
  const [searchedText, setSearchedText] = useState('');
  const [approvedTutorials, setApprovedTutorials] = useState([]);
  const [searching, setSearching] = useState(false);
  const [filteredData, setFilteredData] = useState([]);

  useEffect(() => {
    async function fetchApproved() {
      try {
        const tutorials = await API.get(`/tutorial/categories/1`);
        const { data: tutorialsApproved } = tutorials.data;
        console.log(tutorialsApproved);

        setApprovedTutorials(tutorialsApproved);
      } catch (error) {
        console.warn(error);
      }
    }
    fetchApproved();
  }, [selectedTab]);

  async function handleSearch() {
    if (searchedText.length != 0) {
      setSearching(true);
      const response = await API.get(`tutorial/search/${searchedText}`);
      const { data } = response.data;
      const { status } = data;
      setFilteredData(data);
      if (status === 500) {
        Alert.alert(
          'Falha ao procurar tutoriais',
          'Houve um problema ao tentar buscar tutoriais. Tente novamente mais tarde.',
        );
      }
    }
  }

  async function handleSearchChange(text) {
    if (text.length > 0) {
      setSearchedText(text);
    } else {
      setSearching(false);
    }
  }

  function editTutorial(id) {
    navigation.navigate('EditTutorialInfoScreen', { id: id });
  }

  function removeTutorial(id) {
    Alert.alert(
      'Remover Appoio',
      'Tem certeza que quer remover este Appoio? Esta ação é irreversível',
      [
        { text: 'Cancelar' },
        {
          text: 'Remover',
          style: 'destructive',
          onPress: async e => {
            try {
              await API.delete(`/tutorial/delete/${id}`);
              const tutorials = await API.get(`/tutorial/categories/1`);
              const { data: tutorialsApproved } = tutorials.data;
              setApprovedTutorials(tutorialsApproved);
              if (searching) handleSearch();
            } catch (error) {
              console.warn(error);
            }
            Alert.alert(
              'Removido!',
              'Este appoio foi removido e portanto não será mais apresentado na coleção.',
            );
          },
        },
      ],
    );
  }

  const emptyMessage = (
    text = 'Não existem tutoriais desta categoria ainda.',
  ) => (
    <View style={styles.emptyMessageContainer}>
      <Text style={styles.emptyMessage}>{text}</Text>
    </View>
  );

  return (
    <View style={styles.screen}>
      <HeaderComponent>
        <Image
          style={styles.image}
          source={require('../../assets/logo-p.png')}
        />
        <View style={styles.searchBar}>
          <SearchBarComponent
            handleEnter={() => handleSearch()}
            handleChange={handleSearchChange}
          />
        </View>
      </HeaderComponent>
      <BodyComponent style={{ paddingBottom: 0 }}>
        {searching ? (
          filteredData.length > 0 ? (
            filteredData.map(item => (
              <TutorialCard
                key={item.id}
                action={e =>
                  navigation.navigate('TutorialScreen', {
                    id: item.id,
                  })
                }
                title={item.appoioName}
                author={item.user.name}
                edit={() => editTutorial(item.id)}
                remove={() => removeTutorial(item.id)}
              />
            ))
          ) : (
            emptyMessage('Nenhum tutorial encontrado')
          )
        ) : (
          <View>
            <TabSelectionComponent
              style={{ marginBottom: 10 }}
              tabSelected={selectedTab}
              onPress={index => setSelectedTab(index)}
            >
              <View title="APLICATIVOS" style={styles.content} />
              <View title="CELULAR" style={styles.content} />
              <View title="CONCEITOS" style={styles.content} />
            </TabSelectionComponent>
            {selectedTab == 0 &&
              (approvedTutorials.aplicativos
                ? approvedTutorials.aplicativos.map(item => (
                    <TutorialCard
                      key={item.id}
                      action={e =>
                        navigation.navigate('TutorialScreen', {
                          id: item.id,
                        })
                      }
                      title={item.appoioName}
                      author={item.user.name}
                      edit={() => editTutorial(item.id)}
                      remove={() => removeTutorial(item.id)}
                    />
                  ))
                : emptyMessage())}
            {selectedTab == 1 &&
              (approvedTutorials.celular
                ? approvedTutorials.celular.map(item => (
                    <TutorialCard
                      key={item.id}
                      action={e =>
                        navigation.navigate('TutorialScreen', {
                          id: item.id,
                        })
                      }
                      title={item.appoioName}
                      author={item.user.name}
                      edit={() => editTutorial(item.id)}
                      remove={() => removeTutorial(item.id)}
                    />
                  ))
                : emptyMessage())}
            {selectedTab == 2 &&
              (approvedTutorials.conceitos
                ? approvedTutorials.conceitos.map(item => (
                    <TutorialCard
                      key={item.id}
                      action={e =>
                        navigation.navigate('TutorialScreen', {
                          id: item.id,
                        })
                      }
                      title={item.appoioName}
                      author={item.user.name}
                      edit={() => editTutorial(item.id)}
                      remove={() => removeTutorial(item.id)}
                    />
                  ))
                : emptyMessage())}
          </View>
        )}
      </BodyComponent>
    </View>
  );
}
const styles = StyleSheet.create({
  screen: {
    flex: 1,
    backgroundColor: colors.primary,
  },
  emptyMessage: {
    color: colors.textDisabled,
    fontSize: 20,
    fontWeight: 'bold',
    textAlign: 'center',
    alignSelf: 'center',
  },
  searchBar: {
    marginTop: 20,
  },
  message: {
    marginVertical: 10,
    color: '#000000',
    fontSize: 20,
    fontWeight: 'bold',
    justifyContent: 'center',
  },
  emptyMessageContainer: {
    marginVertical: '10%',
    justifyContent: 'center',
  },
  aplicativos: {
    flexDirection: 'row',
    flexWrap: 'wrap',
    justifyContent: 'space-between',
  },
  image: {
    alignSelf: 'center',
    width: 90,
    height: 46,
  },
});
