/**
 * Tela de modelo com um componente de modelo
 *
 * @author            Joao Leao
 * @example           <AppoioStepScreen/>
 */

import React, { useState } from 'react';
import { StyleSheet, View, Text, Alert } from 'react-native';
import { CommonActions } from '@react-navigation/native';
import Header from '../components/HeaderComponent';
import Body from '../components/BodyComponent';
import Button from '../components/ButtonComponent';
import StepComponent from '../components/StepComponent';
import colors from '../constants/colors';
import { editAppoio } from '../api';
import LoaderComponent from '../components/LoaderComponent';

export default function AppoioStepScreen({ route, navigation }) {
  const [steps, setSteps] = useState([...route.params.data.steps]);
  const [isLoading, setIsLoading] = useState(false);

  async function send() {
    try {
      const data = {
        tutorialId: route.params.data.tutorialId,
        name: route.params.data.name,
        category: route.params.data.chosenOption,
        appId: route.params.data.app,
        appVersion: route.params.data.appVersion,
        os: route.params.data.os,
        osVersion: route.params.data.osVersion,
        tags: route.params.data.tagList,
        steps,
      };

      setIsLoading(true);

      await editAppoio({ data });

      setIsLoading(false);

      Alert.alert('Appoio Editado', '', [
        {
          text: 'OK',
          onPress: () =>
            navigation.dispatch(
              CommonActions.reset({
                index: 0,
                routes: [{ name: 'ApprovedTutorialListScreen' }],
              }),
            ),
        },
      ]);
    } catch (error) {
      setIsLoading(false);
      Alert.alert(error.title, error.message, [{ text: 'OK' }]);
    }
  }

  function isValidData() {
    for (let i = 0; i < steps.length; i++) {
      if (steps[i].description.length == 0) return false;
    }
    return true;
  }
  function handleDescription(text, index) {
    const array = [...steps];
    array[index] = {
      ...array[index],
      description: text,
    };
    setSteps(array);
  }
  function handleVideo(text, index) {
    const array = [...steps];
    array[index] = {
      ...array[index],
      videoURL: text,
    };
    setSteps(array);
  }
  function handleImage(img, index) {
    const array = [...steps];
    const step = array[index];
    array[index] = {
      ...step,
      imgURL: {
        ...img,
        name: `${index}.${img.uri.split('.').pop()}`,
      },
    };
    setSteps(array);
  }

  function handleAdd() {
    setSteps([
      ...steps,
      {
        order: steps.length,
        description: '',
        videoURL: '',
        imgURL: '',
      },
    ]);
  }
  function handleDelete(index) {
    let filtered = steps
      .filter(item => item.order !== index)
      .map(item => {
        if (item.order > index) {
          return {
            ...item,
            order: item.order - 1,
          };
        }
        return item;
      });
    setSteps(filtered);
  }

  return isLoading ? <LoaderComponent /> : (
    <View style={styles.screen}>
      <Header large leftButton leftButtonAction={() => navigation.goBack()}>
        <Text style={styles.title}>
          Altere um <Text style={styles.textBold}>Appoio</Text>
        </Text>
        <Text style={styles.descriptionText}>
          Edite passos sobre o <Text style={styles.textBold}>Appoio</Text> que
          deseja mudar{' '}
        </Text>
      </Header>

      <Body
        buttonAction={send}
        buttonDisabled={!isValidData()}
        buttonText="Próximo"
        buttonBackgroundColor={colors.primary}
        buttonTextColor={colors.textWhite}
        divisor
      >
        {steps.map((item, index) => (
          <StepComponent
            key={index}
            index={index}
            item={item}
            handleDescription={handleDescription}
            handleVideo={handleVideo}
            handleImage={handleImage}
            handleDelete={steps.length > 1 ? handleDelete : null}
          />
        ))}
        <View style={styles.button}>
          <Button
            text="Adicionar Passo"
            bordered
            icon="plus"
            action={handleAdd}
          />
        </View>
      </Body>
    </View>
  );
}
const styles = StyleSheet.create({
  screen: {
    flex: 1,
    backgroundColor: colors.primary,
    alignItems: 'center',
  },
  title: {
    color: colors.textWhite,
    fontSize: 30,
    textAlign: 'center',
  },
  textBold: {
    fontWeight: 'bold',
  },
  descriptionText: {
    color: colors.textWhite,
    fontSize: 22,
    textAlign: 'center',
  },
  button: {
    alignItems: 'center',
  },
});
