/**
 * Tela de exibição de Tutoriais. Ao clicar em um tutorial, mostra seus respectivos passos, além de
 * um componente de Aprovar ou Reprovar tutorial, caso o usuário seja um administrador.
 *
 * @author            Marlon Saldanha, Bernardo de Cesaro e João Leão
 *
 *
 * @example           <TutorialScreen/>
 */

import React from 'react';
import {
  View,
  Text,
  StyleSheet,
  TouchableOpacity,
  Alert,
  AsyncStorage,
} from 'react-native';
import BodyComponent from '../components/BodyComponent';
import HeaderComponent from '../components/HeaderComponent';
import TutorialComponent from '../components/TutorialComponent';
import { CommonActions } from '@react-navigation/native';
import API from '../api';
import colors from '../constants/colors';

function TutorialFeedbackComponent({
  action,
  color,
  disabled,
  text,
  textColor = colors.textWhite,
}) {
  const tutorialButtonStyle = StyleSheet.create({
    buttonStyle: {
      borderColor: textColor,
      backgroundColor: color,
      width: 155,
      height: 50,
      borderRadius: 8,
      borderWidth: 3,
      justifyContent: 'center',
    },
    text: {
      fontSize: 18,
      fontWeight: '500',
      color: textColor,
      textAlign: 'center',
    },
  });
  return (
    <View>
      <TouchableOpacity
        onPress={e => action()}
        disabled={disabled}
        activeOpacity={0.6}
        style={tutorialButtonStyle.buttonStyle}
      >
        <Text style={tutorialButtonStyle.text}>{text}</Text>
      </TouchableOpacity>
    </View>
  );
}

export default function TutorialScreen({ route, navigation }) {
  const [data, setData] = React.useState(null);

  React.useEffect(() => {
    async function fetchData() {
      const { id } = route.params;

      try {
        const response = await API.get(`/tutorial/get/${id}`);
        setData(response.data.data);
      } catch (error) {
        console.warn(error);
      }
    }
    fetchData();
  }, []);

  return (
    <View style={styles.container}>
      <HeaderComponent leftButton leftButtonAction={e => navigation.goBack()}>
        <Text numberOfLines={2} style={styles.title} ellipsizeMode="tail">
          {data?.appoioName}
        </Text>
      </HeaderComponent>
      <BodyComponent>
        {data && <TutorialComponent data={data} />}
        {route.params.isAdmin && (
          <View style={styles.feedbackButtonContainer}>
            <TutorialFeedbackComponent
              color={colors.textPlaceHolder}
              text="REPROVAR"
              action={async () => {
                const { id } = route.params;
                try {
                  const response = await API.delete(`/tutorial/delete/${id}`);
                  setData(response.data.data);
                } catch (error) {
                  console.warn(error);
                }
                Alert.alert(
                  'Reprovado!',
                  'Este appoio foi reprovado e portanto não será incluído na coleção.',
                  [
                    {
                      text: 'Ok',
                      onPress: () => {
                        navigation.dispatch(
                          CommonActions.reset({
                            index: 0,
                            routes: [{ name: 'CuradoriaScreen' }],
                          }),
                        );
                      },
                    },
                  ],
                );
              }}
            />
            <TutorialFeedbackComponent
              color={colors.primary}
              text="APROVAR"
              action={async () => {
                const { id } = route.params;
                try {
                  const response = await API.patch(`/tutorial/approve/${id}`);
                  setData(response.data.data);
                } catch (error) {
                  console.warn(error);
                }
                Alert.alert(
                  'Aprovado!',
                  'Este appoio foi aprovado e já está disponível para os usuários!',
                  [
                    {
                      text: 'Ok',
                      onPress: () => {
                        navigation.dispatch(
                          CommonActions.reset({
                            index: 0,
                            routes: [{ name: 'CuradoriaScreen' }],
                          }),
                        );
                      },
                    },
                  ],
                );
              }}
            />
          </View>
        )}
      </BodyComponent>
    </View>
  );
}

const styles = StyleSheet.create({
  container: {
    flex: 1,
    height: '100%',
    backgroundColor: colors.primary,
    position: 'relative',
  },
  scrollContainer: {
    flex: 1,
    justifyContent: 'center',
  },
  title: {
    color: colors.textWhite,
    fontSize: 30,
    fontWeight: 'bold',
    textAlign: 'center',
  },
  feedbackButtonContainer: {
    flexDirection: 'row',
    justifyContent: 'space-between',
    paddingHorizontal: 25,
    position: 'absolute',
    bottom: 10,
    left: 0,
    right: 0,
  },
});
